@@ -2,13 +2,13 @@ package Mojo::Reactor::POE;
 use Mojo::Base 'Mojo::Reactor::Poll';
 
 use POE;
-use Mojo::Util 'steady_time';
+use Mojo::Util qw(md5_sum steady_time);
 use Scalar::Util 'weaken';
 
 use constant { POE_IO_READ => 0, POE_IO_WRITE => 1 };
 use constant DEBUG => $ENV{MOJO_REACTOR_POE_DEBUG} || 0;
 
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
 my $POE;
 
@@ -38,11 +38,13 @@ sub remove {
 	return unless defined $remove;
 	if (ref $remove) {
 		if (exists $self->{io}{fileno $remove}) {
+			warn "-- Removed IO watcher for ".fileno($remove)."\n" if DEBUG;
 			$self->_send_clear_io(fileno $remove);
 		}
 		return !!delete $self->{io}{fileno $remove};
 	} else {
 		if (exists $self->{timers}{$remove}) {
+			warn "-- Removed timer $remove\n" if DEBUG;
 			$self->_send_clear_timer($remove);
 		}
 		return !!delete $self->{timers}{$remove};
@@ -68,23 +70,34 @@ sub watch {
 	$io->{handle} = $handle;
 	$io->{read} = $read;
 	$io->{write} = $write;
-	$self->_send_set_io(fileno $handle);
 	
 	warn "-- Set IO watcher for ".fileno($handle)."\n" if DEBUG;
 	
+	$self->_send_set_io(fileno $handle);
+	
 	return $self;
 }
 
+sub _id {
+	my $self = shift;
+	my $id;
+	do { $id = md5_sum 't' . steady_time . rand 999 } while $self->{timers}{$id};
+	return $id;
+}
+
 sub _timer {
 	my ($self, $recurring, $after, $cb) = @_;
-	$after ||= 0.0001 if $recurring;
 	
-	my $id = $self->SUPER::_timer($recurring, $after, $cb);
-	my $timer = $self->{timers}{$id};
+	my $id = $self->_id;
+	my $timer = $self->{timers}{$id}
+		= {cb => $cb, after => $after, time => steady_time + $after};
+	$timer->{recurring} = $after if $recurring;
+	
+	my $is_recurring = $recurring ? ' (recurring)' : '';
+	warn "-- Set timer $id after $after seconds$is_recurring\n" if DEBUG;
+	
 	$self->_send_set_timer($id);
 	
-	warn "-- Set timer $id after $after seconds\n" if DEBUG;
-	
 	return $id;
 }
 
@@ -278,8 +291,8 @@ sub _event_timer {
 	
 	my $timer = $self->{timers}{$id};
 	warn "-- Event fired for timer $id\n" if DEBUG;
-	if ($timer->{recurring}) {
-		$timer->{time} = steady_time + $timer->{after};
+	if (exists $timer->{recurring}) {
+		$timer->{time} = steady_time + $timer->{recurring};
 		$self->_send_set_timer($id);
 	} else {
 		delete $self->{timers}{$id};
